<?php
/**
 * @Copyright
 * @package     ESK - Easy Session Killer for Joomla! 3.x
 * @author      Viktor Vogel <admin@kubik-rubik.de>
 * @version     3.1.0 - 2018-02-24
 * @link        https://joomla-extensions.kubik-rubik.de/
 *
 * @license     GNU/GPL
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
defined('_JEXEC') or die('Restricted access');

class PlgSystemEasySessionKiller extends JPlugin
{
	/**
	 * Interval whitelist with correct seconds values
	 *
	 * @var array
	 */
	protected $interval_whitelist = array('daily'    => 86400,
	                                      'bidaily'  => 172800,
	                                      'weekly'   => 604800,
	                                      'biweekly' => 1209600,
	                                      'monthly'  => 2419200);

	/**
	 * PlgSystemEasySessionKiller constructor.
	 *
	 * @param $subject
	 * @param $config
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
	}

	/**
	 * Triggers the cleaning task late in the application process
	 *
	 * @throws Exception
	 */
	public function onAfterRespond()
	{
		if(JFactory::getApplication()->isSite())
		{
			$interval = $this->params->get('interval', 'probability');

			if($interval == 'probability')
			{
				$probability = (int)$this->params->get('probability', 10);

				if(mt_rand(1, 100) <= $probability)
				{
					$this->cleanSessionsTable();

					return;
				}
			}

			if(isset($this->interval_whitelist[$interval]))
			{
				$interval_timestamp = (int)$this->params->get('interval_timestamp', 0);

				if(time() - $interval_timestamp > $this->interval_whitelist[$interval])
				{
					$this->cleanSessionsTable();
					$this->updateIntervalTimestamp();
				}
			}
		}
	}

	/**
	 * Cleans the sessions table
	 */
	private function cleanSessionsTable()
	{
		$session = JFactory::getSession();
		$db = JFactory::getDbo();

		$query = $db->getQuery(true);
		$query->delete($db->quoteName('#__session'));
		$query->where($db->quoteName('time').' < '.$db->quote((int)(time() - $session->getExpire())));
		$db->setQuery($query);

		try
		{
			$db->execute();
		}
		catch(Exception $e)
		{
		}
	}

	/**
	 * Updates the interval timestamp after a cleaning process
	 *
	 * @return bool
	 */
	private function updateIntervalTimestamp()
	{
		$plugin_object = $this->loadPluginDatabase();

		if(!empty($plugin_object))
		{
			$params = json_decode($plugin_object['params']);
			$params->interval_timestamp = time();
			$plugin_object['params'] = json_encode($params);

			$table = JTable::getInstance('extension');
			$table->load($plugin_object['extension_id']);
			$table->bind($plugin_object);

			if(!$table->check())
			{
				return false;
			}

			$table->store();
		}
	}

	/**
	 * Loads the complete table entry of the Easy Session Killer plugin
	 *
	 * @return mixed
	 */
	private function loadPluginDatabase()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('*');
		$query->from('#__extensions');
		$query->where('type = '.$db->quote('plugin'))->where('element = '.$db->quote('easysessionkiller'))->where('folder = '.$db->quote('system'))->where('state IN (0,1)');
		$db->setQuery($query);

		return $db->loadAssoc();
	}
}
